@@ -155,6 +155,8 @@ CAMLprim value ocaml_faad_decode(value _dh, value _inbuf, value _inbufofs, value
 
   if (frameInfo.error > 0)
     caml_raise_with_arg(*caml_named_value("ocaml_faad_exn_error"), Val_int(frameInfo.error));
+  if (!data)
+    caml_raise_constant(*caml_named_value("ocaml_faad_exn_failed"));
 
   outbuf = caml_alloc_tuple(frameInfo.channels);
   for(c = 0; c < frameInfo.channels; c++)
@@ -455,6 +457,8 @@ CAMLprim value ocaml_faad_mp4_init(value m, value dh, value track)
   mp4ff_get_decoder_config(mp->ff, t, &mp4_buffer, &mp4_buffer_size);
   ret = NeAACDecInit2(dec, mp4_buffer, mp4_buffer_size, &samplerate, &channels);
   caml_leave_blocking_section();
+
+  free(mp4_buffer);
   check_err(ret);
 
   ans = caml_alloc_tuple(2);
@@ -515,8 +519,8 @@ CAMLprim value ocaml_faad_mp4_read_sample(value m, value track, value sample)
   mp4_t *mp = Mp4_val(m);
   int t = Int_val(track);
   int s = Int_val(sample);
-  unsigned char *buf;
-  unsigned int buflen;
+  unsigned char *buf = NULL;
+  unsigned int buflen = 0;
   int ret;
 
   caml_enter_blocking_section();
@@ -541,8 +545,8 @@ CAMLprim value ocaml_faad_mp4_decode(value m, value track, value sample, value d
   int s = Int_val(sample);
   NeAACDecHandle dec = Dec_val(dh);
   NeAACDecFrameInfo frameInfo;
-  unsigned char *inbuf;
-  unsigned int inbuflen;
+  unsigned char *inbuf = NULL;
+  unsigned int inbuflen = 0;
   float *data;
   int c, i, ret;
 
